import 'dart:async';
import 'dart:convert';

import 'package:http/http.dart';
import 'package:http_interceptor/http/intercepted_client.dart';
import 'package:http_interceptor/http/interceptor_contract.dart';
import 'package:rec_api_dart/src/api_error.dart';
import 'package:rec_api_dart/src/env.base.dart';
import 'package:rec_api_dart/src/helpers/checks.dart';

/// [API Documentation](https://github.com/QbitArtifacts/rec_app_v2/wiki/Api-Documentation) available.
abstract class ServiceBase {
  late Client client;
  final EnvBase env;

  ServiceBase({
    required this.env,
    List<InterceptorContract> interceptors = const [],
    Client? client,
  }) {
    this.client = client ??
        InterceptedClient.build(
          interceptors: interceptors,
        );
  }

  Future<Map<String, String>> getHeaders({
    String contentType = 'application/json',
    String accept = 'application/json',
  }) {
    return Future.value({
      'content-type': contentType,
      'accept': accept,
    });
  }

  Future<Map<String, dynamic>> put(Uri uri, Map data) async {
    return client
        .put(
          uri,
          headers: await getHeaders(),
          body: json.encode(data),
        )
        .then(onResponse);
  }

  Future<Map<String, dynamic>> post(Uri uri, Map<String, dynamic>? body) async {
    return client
        .post(
          uri,
          headers: await getHeaders(),
          body: json.encode(body),
        )
        .then(onResponse);
  }

  Future delete(Uri url) {
    return client.delete(url).then(onResponse);
  }

  Future<Map<String, dynamic>> get(Uri url) async {
    return client
        .get(
          url,
          headers: await getHeaders(),
        )
        .then(onResponse);
  }

  FutureOr<Map<String, dynamic>> onResponse(Response response) {
    if (response.statusCode >= 400) {
      return Future.error(ApiError.fromResponse(response));
    }

    return Future.value(
      Checks.isNotEmpty(response.body) ? json.decode(response.body) : {},
    );
  }
}
