import 'package:http/http.dart';
import 'package:rec_api_dart/src/api_paths.dart';
import 'package:rec_api_dart/src/entities/document.ent.dart';
import 'package:rec_api_dart/src/entities/document_kind.ent.dart';
import 'package:rec_api_dart/src/entities/dto/create_document_data.dart';
import 'package:rec_api_dart/src/env.base.dart';
import 'package:rec_api_dart/src/interceptors/user_token_interceptor.dart';
import 'package:rec_api_dart/src/interfaces/api_list_response.dart';
import 'package:rec_api_dart/src/services/base_service.dart';

/// This service is in charge of all requests related to [Documents] and [DocumentKinds]
/// Like listing, posting, updating, etc...
///
/// * **Authentication Required** - the user must be authenticated
class DocumentsService extends ServiceBase {
  DocumentsService({required EnvBase env, Client? client})
      : super(
          env: env,
          client: client,
          interceptors: [UserTokenInterceptor()],
        );

  /// List all [DocumentKinds]
  Future<ApiListResponse<DocumentKind>> listKinds() {
    final uri = ApiPaths.documentKinds.withQueryParams({
      'limit': '30',
    }).toUri(env);

    return this.get(uri).then(_mapKinds);
  }

  /// List all [Documents] for the authenticated user
  Future<ApiListResponse<Document>> list() {
    final uri = ApiPaths.documents.toUri(env);

    return this.get(uri).then(_mapDocuments);
  }

  /// Creates a document for the current authenticated user
  Future createDocument(CreateDocumentData data) {
    final uri = ApiPaths.documents.toUri(env);

    return this.post(uri, data.toJson());
  }

  /// Update a document instance
  /// For now, only content can be updated
  Future updateDocument(String? documentId, {required String content}) {
    final uri = ApiPaths.documents.append(documentId).toUri(env);

    return this.put(uri, {'content': content});
  }

  ApiListResponse<DocumentKind> _mapKinds(
    Map<String, dynamic> data,
  ) {
    return ApiListResponse.fromJson(
      data['data'],
      mapper: (el) => DocumentKind.fromJson(el),
    );
  }

  ApiListResponse<Document> _mapDocuments(
    Map<String, dynamic> data,
  ) {
    return ApiListResponse.fromJson(
      {'elements': data['data']},
      mapper: (el) => Document.fromJson(el),
    );
  }
}
