import 'package:http/http.dart';
import 'package:rec_api_dart/src/api_paths.dart';
import 'package:rec_api_dart/src/entities/dto/payment_data.dart';
import 'package:rec_api_dart/src/entities/transactions/day_sales.summary.dart';
import 'package:rec_api_dart/src/entities/transactions/payment_result.dart';
import 'package:rec_api_dart/src/entities/transactions/transaction.ent.dart';
import 'package:rec_api_dart/src/entities/transactions/vendor_data.ent.dart';
import 'package:rec_api_dart/src/env.base.dart';
import 'package:rec_api_dart/src/interceptors/user_token_interceptor.dart';
import 'package:rec_api_dart/src/interfaces/api_list_response.dart';
import 'package:rec_api_dart/src/services/base_service.dart';

/// This service is in charge of all requests related to [Transaction] and payments
///
/// * **Authentication Required** - the user must be authenticated
class TransactionsService extends ServiceBase {
  TransactionsService({required EnvBase env, Client? client})
      : super(
          env: env,
          client: client,
          interceptors: [UserTokenInterceptor()],
        );

  /// list all transactions, at an offset with a limit
  ///
  /// `offset` the page number
  /// `limit` the amount of items per page
  Future<ApiListResponse<Transaction>> list({
    int offset = 0,
    int limit = 10,
    Map<String, dynamic> extraParams = const {},
  }) {
    final params = {
      'offset': '$offset',
      'limit': '$limit',
      ...extraParams,
    };

    final uri = ApiPaths.transactions.withQueryParams(params).toUri(env);

    return this.get(uri).then(_mapToApiListReponse);
  }

  /// List all transactions for a given date
  Future<DaySalesSummary> listForDate({required DateTime date}) {
    final params = {'day': '${date.year}-${date.month}-${date.day}'};
    final uri = ApiPaths.transactionsPerDay.withQueryParams(params).toUri(env);

    return this.get(uri).then(_mapSummary);
  }

  /// Gets information about a given address, like name, image and type
  Future<VendorData> getVendorInfoFromAddress(String? address) {
    final uri = ApiPaths.vendorData.withQueryParams({
      'address': address,
    }).toUri(env);

    return this.get(uri).then((resp) => VendorData.fromJson(resp['data']));
  }

  /// Make a payment to an Account from the current account
  Future<PaymentResult> makePayment(PaymentData data) {
    final uri = ApiPaths.payment.toUri(env);

    return this
        .post(uri, data.toJson())
        .then((resp) => PaymentResult.fromJson(resp));
  }

  // Maps api list reponse to an instance of ApiListResponse<Transaction>
  ApiListResponse<Transaction> _mapToApiListReponse(Map<String, dynamic> data) {
    return ApiListResponse<Transaction>.fromJson(
      data['data'],
      mapper: (el) {
        return Transaction.fromJson(el);
      },
    );
  }

  DaySalesSummary _mapSummary(Map<String, dynamic> data) {
    return DaySalesSummary.fromJson(data['data']);
  }
}
